@@ -1,4 +1,9 @@
+import io
 import json
+from datetime import datetime
+from urllib import parse
+from uuid import UUID
+
 import requests
 
 from .constants import ROW_FILTER_KEYS, ColumnTypes
@@ -27,6 +32,7 @@ def parse_response(response):
         except:
             pass
 
+
 class SeaTableAPI(object):
     """SeaTable API
     """
@@ -42,6 +48,8 @@ class SeaTableAPI(object):
         self.dtable_uuid = None
         self.headers = None
         self.socketIO = None
+        self.workspace_id = None
+        self.dtable_name = None
 
     def __str__(self):
         return 'SeaTableAPI Object [ %s ]' % self.dtable_uuid
@@ -58,6 +66,8 @@ class SeaTableAPI(object):
         jwt_token = data.get('access_token')
         self.headers = parse_headers(jwt_token)
         self.dtable_server_url = parse_server_url(data.get('dtable_server'))
+        self.workspace_id = data.get('workspace_id')
+        self.dtable_name = data.get('dtable_name')
 
         if with_socket_io is True:
             self.socketIO = connect_socket_io(
@@ -428,3 +438,193 @@ class SeaTableAPI(object):
         response = requests.delete(url, json=json_data, headers=self.headers)
         data = parse_response(response)
         return data
+
+    def download_file(self, url, save_path):
+        if not str(UUID(self.dtable_uuid)) in url:
+            raise Exception('url invalid.')
+        path = url.split(str(UUID(self.dtable_uuid)))[-1].strip('/')
+        download_link = self.get_file_download_link(parse.unquote(path))
+        response = requests.get(download_link)
+        if response.status_code != 200:
+            raise Exception('download file error')
+        with open(save_path, 'wb') as f:
+            f.write(response.content)
+
+    def upload_bytes_file(self, name, content: bytes, relative_path=None, file_type=None, replace=False):
+        """
+        relateive_path: relative path for upload, if None, default {file_type}s/{date of this month} eg: files/2020-09
+        file_type: if relative is None, file type must in ['image', 'file'], default 'file'
+        return: info dict of uploaded file
+        """
+        upload_link_dict = self.get_file_upload_link()
+        parent_dir = upload_link_dict['parent_path']
+        upload_link = upload_link_dict['upload_link'] + '?ret-json=1'
+        if not relative_path:
+            if file_type and file_type not in ['image', 'file']:
+                raise Exception('relative or file_type invalid.')
+            if not file_type:
+                file_type = 'file'
+            relative_path = '%ss/%s' % (file_type, str(datetime.today())[:7])
+        else:
+            relative_path = relative_path.strip('/')
+        response = requests.post(upload_link, data={
+            'parent_dir': parent_dir,
+            'relative_path': relative_path,
+            'replace': 1 if replace else 0
+        }, files={
+            'file': (name, io.BytesIO(content))
+        })
+        d = response.json()[0]
+        url = '%(server)s/workspace/%(workspace_id)s/asset/%(dtable_uuid)s/%(relative_path)s/%(filename)s' % {
+            'server': self.server_url.strip('/'),
+            'workspace_id': self.workspace_id,
+            'dtable_uuid': str(UUID(self.dtable_uuid)),
+            'file_type': file_type,
+            'relative_path': parse.quote(relative_path.strip('/')),
+            'filename': parse.quote(d.get('name', name))
+        }
+        return {
+            'type': file_type,
+            'size': d.get('size'),
+            'name': d.get('name'),
+            'url': url
+        }
+
+    def upload_local_file(self, file_path, name=None, relative_path=None, file_type=None, replace=False):
+        """
+        relateive_path: relative path for upload, if None, default {file_type}s/{date of today}, eg: files/2020-09
+        file_type: if relative is None, file type must in ['image', 'file'], default 'file'
+        return: info dict of uploaded file
+        """
+        if file_type not in ['image', 'file']:
+            raise Exception('file_type invalid.')
+        if not name:
+            name = file_path.strip('/').split('/')[-1]
+        if not relative_path:
+            if file_type and file_type not in ['image', 'file']:
+                raise Exception('relative or file_type invalid.')
+            if not file_type:
+                file_type = 'file'
+            relative_path = '%ss/%s' % (file_type, str(datetime.today())[:7])
+        else:
+            relative_path = relative_path.strip('/')
+        upload_link_dict = self.get_file_upload_link()
+        parent_dir = upload_link_dict['parent_path']
+        upload_link = upload_link_dict['upload_link'] + '?ret-json=1'
+        response = requests.post(upload_link, data={
+            'parent_dir': parent_dir,
+            'relative_path': relative_path,
+            'replace': 1 if replace else 0
+        }, files={
+            'file': (name, open(file_path, 'rb'))
+        })
+        d = response.json()[0]
+        url = '%(server)s/workspace/%(workspace_id)s/asset/%(dtable_uuid)s/%(relative_path)s/%(filename)s' % {
+            'server': self.server_url.strip('/'),
+            'workspace_id': self.workspace_id,
+            'dtable_uuid': str(UUID(self.dtable_uuid)),
+            'file_type': file_type,
+            'relative_path': parse.quote(relative_path.strip('/')),
+            'filename': parse.quote(d.get('name', name))
+        }
+        return {
+            'type': file_type,
+            'size': d.get('size'),
+            'name': d.get('name'),
+            'url': url
+        }
+
+class Account(object):
+    def __init__(self, login_name, password, server_url):
+        self.login_name = login_name
+        self.username = None
+        self.password = password
+        self.server_url = server_url.strip().strip('/')
+        self.token = None
+
+    def __str__(self):
+        return 'Account Object [%s]' % (self.login_name)
+
+    def _get_api_token_url(self):
+        return '%s/api2/auth-token/' % (self.server_url,)
+
+    def _list_workspaces_url(self):
+        return '%s/api/v2.1/workspaces/' % (self.server_url,)
+
+    def _add_base_url(self):
+        return '%s/api/v2.1/dtables/' % (self.server_url,)
+
+    def _get_account_info_url(self):
+        return '%s/api2/account/info/' % (self.server_url,)
+
+    def _get_copy_dtable_url(self):
+        return '%s/api/v2.1/dtable-copy/' % (self.server_url,)
+
+    def _get_temp_api_token_url(self, workspace_id, name):
+        return '%(server_url)s/api/v2.1/workspace/%(workspace_id)s/dtable/%(name)s/temp-api-token/' % {
+            'server_url': self.server_url,
+            'workspace_id': workspace_id,
+            'name': name
+        }
+
+    @property
+    def token_headers(self):
+        return {
+            'Authorization': 'Token %s' % (self.token,)
+        }
+
+    def auth(self):
+        response = requests.post(self._get_api_token_url(), data={
+            'username': self.login_name,
+            'password': self.password
+        })
+        data = parse_response(response)
+        self.token = data.get('token')
+
+    def load_account_info(self):
+        response = requests.get(self._get_account_info_url(), headers=self.token_headers)
+        self.username = parse_response(response).get('email')
+
+    def list_workspaces(self):
+        response = requests.get(self._list_workspaces_url(), headers=self.token_headers)
+        return parse_response(response)
+
+    def add_base(self, name, workspace_id=None):
+        owner = None
+        if not workspace_id:
+            if not self.username:
+                self.load_account_info()  # load username for owner
+            owner = self.username
+        else:
+            workspace_list = self.list_workspaces()['workspace_list']
+            for w in workspace_list:
+                if w.get('id') == workspace_id and w.get('group_id'):
+                    owner = '%s@seafile_group' % w['group_id']
+                    break
+                if w.get('id') == workspace_id and w.get('owner_type') == 'Personal':
+                    if not self.username:
+                        self.load_account_info()  # load username for owner
+                    owner = self.username
+                    break
+        if not owner:
+            raise Exception('workspace_id invalid.')
+        response = requests.post(self._add_base_url(), data={
+            'name': name,
+            'owner': owner
+        }, headers=self.token_headers)
+        return parse_response(response).get('table')
+
+    def copy_base(self, src_workspace_id, base_name, dst_workspace_id):
+        response = requests.post(self._get_copy_dtable_url(), data={
+            'src_workspace_id': src_workspace_id,
+            'name': base_name,
+            'dst_workspace_id': dst_workspace_id
+        }, headers=self.token_headers)
+        return parse_response(response).get('dtable')
+
+    def get_base(self, workspace_id, base_name, with_socket_io=False):
+        response = requests.get(self._get_temp_api_token_url(workspace_id, base_name), headers=self.token_headers)
+        api_token = parse_response(response).get('api_token')
+        base = SeaTableAPI(api_token, self.server_url)
+        base.auth(with_socket_io=with_socket_io)
+        return base
